﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

// Be sure to set these BackgroundWorker properties at design time:
//      WorkerReportsProgress = true
//      WorkerSupportsCancellation = true

namespace howto_watermark_directory
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The original watermark.
        private Bitmap Watermark;

        // Load the input and output directories.
        private void Form1_Load(object sender, EventArgs e)
        {
            txtInput.Text = Properties.Settings.Default.InputDirectory;
            txtOutput.Text = Properties.Settings.Default.OutputDirectory;
            Watermark = (Bitmap)picWatermark.Image;

            // Clear the file name label.
            lblFile.Text = "";
        }

        // Save the input and output directories.
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.InputDirectory = txtInput.Text;
            Properties.Settings.Default.OutputDirectory = txtOutput.Text;
            Properties.Settings.Default.Save();
        }

        // Let the user pick a new watermark file.
        private void btnSelectWatermark_Click(object sender, EventArgs e)
        {
            btnSelectWatermark.Text = "";
            if (ofdWatermark.ShowDialog() == DialogResult.OK)
            {
                Watermark = new Bitmap(ofdWatermark.FileName);
                picWatermark.Image = Watermark;
            }
        }

        // Let the user select the input folder.
        private void btnInput_Click(object sender, EventArgs e)
        {
            fbdDirectory.SelectedPath = txtInput.Text;
            if (fbdDirectory.ShowDialog() == DialogResult.OK)
            {
                txtInput.Text = fbdDirectory.SelectedPath;
            }
        }

        // Let the user select the output folder.
        private void btnOutput_Click(object sender, EventArgs e)
        {
            fbdDirectory.SelectedPath = txtOutput.Text;
            if (fbdDirectory.ShowDialog() == DialogResult.OK)
            {
                txtOutput.Text = fbdDirectory.SelectedPath;
            }
        }

        // Start or stop adding watermarks.
        private void btnProcessFiles_Click(object sender, EventArgs e)
        {
            if (btnProcessFiles.Text == "Process Files")
            {
                // Launch the BackgroundWorker.
                btnProcessFiles.Text = "Stop";
                Cursor = Cursors.WaitCursor;
                bwProcessImage.RunWorkerAsync();
            }
            else
            {
                // Stop.
                btnProcessFiles.Text = "Process Files";
                bwProcessImage.CancelAsync();
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Set the image's opacity.
        private Bitmap SetOpacity(Bitmap input_bm, float opacity)
        {
            // Make the new bitmap.
            Bitmap output_bm = new Bitmap(
                input_bm.Width, input_bm.Height);

            // Make an associated Grpahics object.
            using (Graphics gr = Graphics.FromImage(output_bm))
            {
                // Make a ColorMatrix with the opacity.
                ColorMatrix color_matrix = new ColorMatrix();
                color_matrix.Matrix33 = opacity;

                // Make the ImageAttributes object.
                ImageAttributes attributes = new ImageAttributes();
                attributes.SetColorMatrix(color_matrix,
                    ColorMatrixFlag.Default, ColorAdjustType.Bitmap);

                // Draw the input biitmap onto the Graphcis object.
                Rectangle rect = new Rectangle(0, 0,
                    output_bm.Width, output_bm.Height);

                gr.DrawImage(input_bm, rect,
                    0, 0, input_bm.Width, input_bm.Height,
                    GraphicsUnit.Pixel, attributes);
            }
            return output_bm;
        }

        // Add the watermark to the files in the background.
        private int NumProcessed = 0;
        private void bwProcessImage_DoWork(object sender, DoWorkEventArgs e)
        {
            // Get parameters.
            int file_width = int.Parse(txtImageWidth.Text);
            int file_height = int.Parse(txtImageHeight.Text);
            int wm_width = int.Parse(txtWatermarkWidth.Text);
            int wm_height = int.Parse(txtWatermarkHeight.Text);
            int xmargin = int.Parse(txtMarginX.Text);
            int ymargin = int.Parse(txtMarginY.Text);
            float opacity = float.Parse(txtOpacity.Text);

            string output_path = txtOutput.Text;
            if (!output_path.EndsWith("\\")) output_path += "\\";

            // Adjust the watermark's opacity.
            Bitmap wm = SetOpacity(Watermark, opacity);

            // Get the watermark's input rectangle.
            RectangleF source_rect = new RectangleF(
                0, 0, wm.Width, wm.Height);

            // Loop through the files.
            NumProcessed = 0;
            FileInfo[] file_infos = null;
            try
            {
                DirectoryInfo input_dir_info = new DirectoryInfo(txtInput.Text);
                file_infos = input_dir_info.GetFiles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                e.Cancel = true;
                return;
            }

            foreach (FileInfo input_file_info in file_infos)
            {
                string filename = Path.Combine(output_path,
                    input_file_info.Name);

                Bitmap bm = null;
                try
                {
                    // Load the input file.
                    bm = new Bitmap(input_file_info.FullName);

                    // Get the scale.
                    float xscale = file_width / (float)bm.Width;
                    float yscale = file_height / (float)bm.Height;
                    float scale = Math.Min(xscale, yscale);

                    // Make a destination rectangle so the watermark
                    // has the desired size when the image is scaled.
                    RectangleF dest_rect = new RectangleF(
                        xmargin, ymargin,
                        wm_width / scale, wm_height / scale);

                    // Draw the watermark on the image.
                    using (Graphics gr = Graphics.FromImage(bm))
                    {
                        gr.InterpolationMode =
                            InterpolationMode.HighQualityBicubic;
                        gr.DrawImage(wm, dest_rect, source_rect,
                            GraphicsUnit.Pixel);
                    }

                    // Save the result.
                    SaveImage(bm, filename);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Skipped {0}. {1}",
                        input_file_info.Name, ex.Message);
                }

                // Show progress.
                NumProcessed++;
                int percent_complete =
                    (100 * NumProcessed) / file_infos.Length;
                Progress progress = new Progress(bm, filename);
                bwProcessImage.ReportProgress(
                    percent_complete, progress);

                // See if we should cancel.
                if (bwProcessImage.CancellationPending)
                {
                    e.Cancel = true;
                    break;
                }
            }
        }

        // Update the progress bar.
        private void bwProcessImage_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            prgFiles.Value = e.ProgressPercentage;
            Progress progress = (Progress)e.UserState;
            lblFile.Text = progress.Filename;
            picWatermark.Image = progress.Image;
            picWatermark.Refresh();
            Console.WriteLine("Saved file " + progress.Filename);
        }

        // Clean up.
        private void bwProcessImage_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            prgFiles.Value = 0;
            lblFile.Text = "";
            picWatermark.Image = Watermark;
            Cursor = Cursors.Default;
            MessageBox.Show("Processed " +
                NumProcessed.ToString() + " files");
            btnProcessFiles.Text = "Process Files";
        }
    }
}
